# -*- coding: utf-8 -*-
import re, time
import numpy as np

import zhinst.ziPython as ziPython
import zhinst.utils as utils

def run_example(port=None, api_level=None, amplitude=0.1, do_plot=False):
    """
    Perform a frequency sweep and record demodulator data using ziPython's
    ziDAQSweeper module.

    Additional configuration: Connect signal output 1 to signal input 1 with a
    BNC cable.
    """
    # create an instance of ziDAQServer: connect to the server
    daq = utils.autoConnect(port, api_level)
    # find an attached instrument
    device = utils.autoDetect(daq)

    # find out whether the device is an HF2 or a UHF
    devtype = daq.getByte('/%s/features/devtype' %device)
    options = daq.getByte('/%s/features/options' %device)

    # configure the device for the sweep
    out_c = 0
    in_c = 0
    demod_c = 0
    osc_c = 0
    demod_rate = 10e3
    if re.match('UHF',devtype) and (not re.match('MF',options)):
        out_mixer_c = 3
    elif re.match('HF2',devtype) and (not re.match('MF',options)):
        out_mixer_c = 6
    else:
        out_mixer_c = 0

    # create a base configuration: disable all outputs, demods and scopes
    general_setting = [
         ['/%s/demods/*/rate' %device, 0],
         ['/%s/demods/*/trigger' %device, 0],
         ['/%s/sigouts/*/enables/*' %device, 0]
    ]
    # if the device is a UHF additionally disable all demodulators
    if re.match('UHF',devtype):
        general_setting.append( ['/%s/demods/*/enable' %device, 0] )
        general_setting.append( ['/%s/scopes/*/enable' %device, 0] )
    elif re.match('HF2',devtype):
        general_setting.append( ['/%s/scopes/*/trigchannel' %device, -1 ])
    daq.set(general_setting)

    # now set the settings relevant to this experiment
    exp_setting = [
        ['/%s/sigins/%d/imp50' %(device,in_c), 1],
        ['/%s/sigins/%d/ac'    %(device,in_c), 0],
        ['/%s/sigins/%d/range' %(device,in_c), 2*amplitude],
        
        ['/%s/demods/%d/order'        %(device,demod_c), 4],
        ['/%s/demods/%d/rate'         %(device,demod_c), demod_rate],
        ['/%s/demods/%d/harmonic'     %(device,demod_c), 1],
        ['/%s/demods/%d/phaseshift'   %(device,demod_c), 0],

        ['/%s/sigouts/%d/on'            %(device,out_c), 1],
        ['/%s/sigouts/%d/range'         %(device,out_c), 1],
        ['/%s/sigouts/%d/enables/%d'    %(device,out_c,out_mixer_c), 1],
        ['/%s/sigouts/%d/amplitudes/%d' %(device,out_c,out_mixer_c), amplitude]
    ]
    # device dependent configuration
    if re.match('UHF',devtype):
        exp_setting.append( ['/%s/demods/%d/enable' %(device,demod_c), 1], )
        if re.match('.*MF',options):
            exp_setting.append( ['/%s/demods/%d/oscselect' %(device,demod_c), osc_c] )
            exp_setting.append( ['/%s/demods/%d/adcselect' %(device,demod_c), in_c] )
    elif re.match('HF2',devtype):
        exp_setting.append( ['/%s/demods/%d/oscselect' %(device,demod_c), osc_c] )
        exp_setting.append( ['/%s/demods/%d/adcselect' %(device,demod_c), in_c] )
        exp_setting.append( ['/%s/sigins/%d/diff' %(device,in_c), 0] )
        exp_setting.append( ['/%s/sigouts/%d/add'  %(device,out_c), 0] )
    daq.set(exp_setting);

    # create an instance of the sweeper class with samplecount grid nodes,
    # here we will sweep over samplecount frequencies
    samplecount = 100
    timeout = 500
    sweeper = daq.sweep(timeout)
    # set sweep parameters
    sweeper.set('sweep/device', device)
    sweeper.set('sweep/start', 4e3)
    sweeper.set('sweep/stop', 50e6)
    # the node we will sweep over, start and stop values
    sweeper.set('sweep/gridnode', 'oscs/%d/freq' %osc_c)
    # specify logarithmic spacing for the gridnode
    sweeper.set('sweep/xmapping', 1)
    # automatically control bandwidth/timeconstants
    sweeper.set('sweep/bandwidthcontrol', 0)
    # sequential scanning (as opposed to binary or bidirectional)
    sweeper.set('sweep/scan', 0)
    # set the sweeper's samplecount
    sweeper.set('sweep/samplecount', samplecount)
    # specify the number of sweeps to perform
    loopcount = 2
    sweeper.set('sweep/loopcount', loopcount)
    sweeper.set('sweep/averaging/tc', 10) # 5*tc
    # number of samples to average over
    sweeper.set('sweep/averaging/sample', 10)

    # subscribe to the nodes from which data will be recorded
    path = '/%s/demods/%d/sample' %(device,demod_c)
    sweeper.subscribe(path)

    # start the sweep
    sweeper.execute()

    start = time.time()
    timeout = 60 # [s]
    print "Will perform %d sweeps...." %loopcount
    while not sweeper.finished():
        time.sleep(0.2)
        p = sweeper.progress()
        print "Individual sweep %.2f%% complete.   \r" %(100*p),
        ## here we could read intermediate data via 
        # data = sweeper.read()...
        ## and process it
        # if device in data:
        # ...
        if (time.time() - start) > timeout :
            # if for some reason the sweep is blocking force the end of the
            # measurement
            print "\nSweep still not finished, forcing finish..."
            sweeper.finish()
    print ""

    # read the sweep data. this command can also be executed whilst sweeping
    # (before finished() is True), in this case sweep data up to that time
    # point is returned. it's still necessary still need to issue read() at
    # the end to fetch the rest.
    data = sweeper.read()
    sweeper.unsubscribe(path)
    
    # stop the sweeper thread and clear the memory
    sweeper.clear()

    # check the dictionary returned is non-empty
    assert data, "read() returned an empty data dictionary, did you subscribe to any paths?"
    # note: data could be empty if no data arrived, e.g., if the demods were
    # disabled or had rate 0
    assert device in data, "data dictionary has no key '%s'" %(device)
    samples = data[device]['demods'][str(demod_c)]['sample']
    print "sample contains %d sweeps" %len(samples)

    if do_plot:
        import matplotlib
        import matplotlib.pyplot as plt
        fig = plt.figure()
        plt.hold(True)
        for i in range(0,len(samples)):
            # please note: the "[i][0]" indexing is known issue to be fixed in
            # an upcoming release (there shouldn't be an additional [0])
            frequency = samples[i][0]['frequency']
            R = np.sqrt(samples[i][0]['x']**2 + samples[i][0]['y']**2)
            plt.loglog(frequency,R)
        plt.grid(True)
        plt.title('Results of %d sweeps' %len(samples))
        plt.xlabel('Frequency (Hz)')
        plt.ylabel('Amplitude (V_RMS)')
        plt.autoscale()
        plt.draw()
        plt.show()

    return samples

if __name__ == '__main__':
    sample = run_example()
